﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Encodings.RealVectorEncoding;

namespace HeuristicLab.Problems.Programmable {
  public sealed class ParameterVector : IDeepCloneable {
    private Dictionary<string, BinaryVector> BinaryParameters;
    private Dictionary<string, IntegerVector> IntegerParameters;
    private Dictionary<string, RealVector> RealParameters;
    private Dictionary<string, Permutation> PermutationParameters;

    public ParameterVector(IEnumerable<KeyValuePair<string, BinaryVector>> binaryVectors = null,
      IEnumerable<KeyValuePair<string, IntegerVector>> integerVectors = null,
      IEnumerable<KeyValuePair<string, RealVector>> realVectors = null,
      IEnumerable<KeyValuePair<string, Permutation>> permutations = null) {
      if (binaryVectors != null) BinaryParameters = binaryVectors.ToDictionary(x => x.Key, x => x.Value);
      if (integerVectors != null) IntegerParameters = integerVectors.ToDictionary(x => x.Key, x => x.Value);
      if (realVectors != null) RealParameters = realVectors.ToDictionary(x => x.Key, x => x.Value);
      if (permutations != null) PermutationParameters = permutations.ToDictionary(x => x.Key, x => x.Value);
    }
    private ParameterVector(ParameterVector original, Cloner cloner) {
      cloner.RegisterClonedObject(original, this);
      if (original.BinaryParameters != null) {
        BinaryParameters = new Dictionary<string, BinaryVector>(original.BinaryParameters.Comparer);
        foreach (var param in original.BinaryParameters)
          BinaryParameters[param.Key] = cloner.Clone(param.Value);
      }
      if (original.IntegerParameters != null) {
        IntegerParameters = new Dictionary<string, IntegerVector>(original.IntegerParameters.Comparer);
        foreach (var param in original.IntegerParameters)
          IntegerParameters[param.Key] = cloner.Clone(param.Value);
      }
      if (original.RealParameters != null) {
        RealParameters = new Dictionary<string, RealVector>(original.RealParameters.Comparer);
        foreach (var param in original.RealParameters)
          RealParameters[param.Key] = cloner.Clone(param.Value);
      }
      if (original.PermutationParameters != null) {
        PermutationParameters = new Dictionary<string, Permutation>(original.PermutationParameters.Comparer);
        foreach (var param in original.PermutationParameters)
          PermutationParameters[param.Key] = cloner.Clone(param.Value);
      }
    }

    public object Clone() {
      return Clone(new Cloner());
    }

    public IDeepCloneable Clone(Cloner cloner) {
      return new ParameterVector(this, cloner);
    }

    public BinaryVector Binary(string name) {
      return BinaryParameters[name];
    }

    public bool Binary(string name, int index) {
      return BinaryParameters[name][index];
    }

    public IEnumerable<string> BinaryNames {
      get { return BinaryParameters != null ? BinaryParameters.Keys : Enumerable.Empty<string>(); }
    }

    public IntegerVector Integer(string name) {
      return IntegerParameters[name];
    }

    public int Integer(string name, int index) {
      return IntegerParameters[name][index];
    }

    public IEnumerable<string> IntegerNames {
      get { return IntegerParameters != null ? IntegerParameters.Keys : Enumerable.Empty<string>(); }
    }

    public RealVector Real(string name) {
      return RealParameters[name];
    }

    public double Real(string name, int index) {
      return RealParameters[name][index];
    }

    public IEnumerable<string> RealNames {
      get { return RealParameters != null ? RealParameters.Keys : Enumerable.Empty<string>(); }
    }

    public Permutation Permutation(string name) {
      return PermutationParameters[name];
    }

    public IEnumerable<string> PermutationNames {
      get { return PermutationParameters != null ? PermutationParameters.Keys : Enumerable.Empty<string>(); }
    }
  }
}
